@@ -1,4 +1,4 @@
-# vim: ts=2 sw=2 sts=0 noexpandtab:
+# vim: ts=8 sw=2 sts=0 noexpandtab:
 ##########################################################
 ## This script is part of the Devel::NYTProf distribution
 ##
@@ -7,11 +7,11 @@
 ## http://search.cpan.org/dist/Devel-NYTProf/
 ##
 ###########################################################
-## $Id: NYTProf.pm 497 2008-10-08 22:34:59Z tim.bunce $
+## $Id: NYTProf.pm 583 2008-11-01 08:06:13Z tim.bunce $
 ###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '2.05';
+our $VERSION = '2.07';
 
 package    # hide the package from the PAUSE indexer
     DB;
@@ -20,7 +20,7 @@ package    # hide the package from the PAUSE indexer
 # Set the flags that influence compilation ASAP so we get full details
 # (sub line ranges etc) of modules loaded as a side effect of loading
 # Devel::NYTProf::Core (ie XSLoader, strict, Exporter etc.)
-# See "perldoc perlvar" for details of the $^P flags
+# See "perldoc perlvar" for details of the $^P ($PERLDB) flags
 $^P = 0x010     # record line range of sub definition
     | 0x100     # informative "file" names for evals
     | 0x200;    # informative names for anonymous subroutines
@@ -28,8 +28,8 @@ $^P = 0x010     # record line range of sub definition
 # XXX hack, need better option handling
 my $use_db_sub = ($ENV{NYTPROF} && $ENV{NYTPROF} =~ m/\buse_db_sub=1\b/);
 
-$^P |= 0x002    # line-by-line profiling (if $DB::single true)
-    | 0x020     # start (after BEGINs) with single-step on
+$^P |= 0x002    # line-by-line profiling via DB::DB (if $DB::single true)
+    |  0x020    # start (after BEGINs) with single-step on
     if $use_db_sub;
 
 require Devel::NYTProf::Core;    # loads XS
@@ -52,36 +52,90 @@ Devel::NYTProf - Powerful feature-rich perl source code profiler
 
 =head1 SYNOPSIS
 
- # profile code and write database to ./nytprof.out
- perl -d:NYTProf some_perl.pl
+  # profile code and write database to ./nytprof.out
+  perl -d:NYTProf some_perl.pl
 
- # convert database into a set of html files, e.g., ./nytprof/index.html
- nytprofhtml
+  # convert database into a set of html files, e.g., ./nytprof/index.html
+  nytprofhtml
 
- # or into comma seperated files, e.g., ./nytprof/*.csv
- nytprofcsv
+  # or into comma seperated files, e.g., ./nytprof/*.csv
+  nytprofcsv
 
 =head1 DESCRIPTION
 
 Devel::NYTProf is a powerful feature-rich perl source code profiler.
 
- * Performs per-line statement profiling for fine detail
- * Performs per-subroutine statement profiling for overview
- * Performs per-block statement profiling (the first profiler to do so)
- * Accounts correctly for time spent after calls return
- * Performs inclusive and exclusive timing of subroutines
- * Subroutine times are per calling location (a powerful feature)
- * Can profile compile-time activity, just run-time, or just END time
- * Uses novel techniques for efficient profiling
- * Sub-microsecond (100ns) resolution on systems with clock_gettime()
- * Very fast - the fastest statement and subroutine profilers for perl
- * Handles applications that fork, with no performance cost
- * Immune from noise caused by profiling overheads and I/O
- * Program being profiled can stop/start the profiler
- * Generates richly annotated and cross-linked html reports
- * Trivial to use with mod_perl - add one line to httpd.conf
- * Includes an extensive test suite
- * Tested on very large codebases
+=over
+
+=item *
+
+Performs per-line statement profiling for fine detail
+
+=item *
+
+Performs per-subroutine statement profiling for overview
+
+=item *
+
+Performs per-block statement profiling (the first profiler to do so)
+
+=item *
+
+Accounts correctly for time spent after calls return
+
+=item *
+
+Performs inclusive and exclusive timing of subroutines
+
+=item *
+
+Subroutine times are per calling location (a powerful feature)
+
+=item *
+
+Can profile compile-time activity, just run-time, or just END time
+
+=item *
+
+Uses novel techniques for efficient profiling
+
+=item *
+
+Sub-microsecond (100ns) resolution on systems with clock_gettime()
+
+=item *
+
+Very fast - the fastest statement and subroutine profilers for perl
+
+=item *
+
+Handles applications that fork, with no performance cost
+
+=item *
+
+Immune from noise caused by profiling overheads and I/O
+
+=item *
+
+Program being profiled can stop/start the profiler
+
+=item *
+
+Generates richly annotated and cross-linked html reports
+
+=item *
+
+Trivial to use with mod_perl - add one line to httpd.conf
+
+=item *
+
+Includes an extensive test suite
+
+=item *
+
+Tested on very large codebases
+
+=back
 
 NYTProf is effectively two profilers in one: a statement profiler, and a
 subroutine profiler.
@@ -138,9 +192,13 @@ Subroutine entry is detected by intercepting the entersub opcode. Subroutine
 exit is detected via perl's internal save stack. The result is both extremely
 fast and very robust.
 
-Note that subroutines that recurse directly or indirectly, such as Error::try,
-will show higher subroutine inclusive times because the time spent recuring
-will be double-counted. That may change in future.
+=head3 Subroutine Recursion
+
+For subroutines that recurse directly or indirectly, such as Error::try,
+the inclusive time is only measured for the outer-most call.
+
+The inclusive times of recursive calls are still measured and are accumulated
+separately. Also the 'maximum recursion depth' per calling location is recorded.
 
 =head2 Application Profiling
 
@@ -162,7 +220,7 @@ taken to ensure it's very efficient.
 
 Just add one line near the start of your httpd.conf file:
 
-	PerlModule Devel::NYTProf::Apache
+  PerlModule Devel::NYTProf::Apache
 
 By default you'll get a F</tmp/nytprof.$$.out> file for the parent process and
 a F</tmp/nytprof.$parent.out.$$> file for each worker process.
@@ -177,15 +235,15 @@ Works with mod_perl 1 and 2. See L<Devel::NYTProf::Apache> for more information.
 
 Usually you'd load Devel::NYTProf on the command line using the perl -d option:
 
- perl -d:NYTProf some_perl.pl
+  perl -d:NYTProf some_perl.pl
 
 To save typing the ':NYTProf' you could set the PERL5DB env var 
 
- PERL5DB='use Devel::NYTProf'
+  PERL5DB='use Devel::NYTProf'
 
 and then just perl -d would work:
 
- perl -d some_perl.pl
+  perl -d some_perl.pl
 
 Or you can avoid the need to add the -d option at all by using the C<PERL5OPT> env var:
 
@@ -201,7 +259,7 @@ environment variable C<NYTPROF>.  It is possible to use this environment
 variable to effect multiple setting by separating the values with a C<:>.  For
 example:
 
-    export NYTPROF=trace=2:start=init:file=/tmp/nytprof.out
+  export NYTPROF=trace=2:start=init:file=/tmp/nytprof.out
 
 =head2 addpid=1
 
@@ -227,15 +285,24 @@ by calling DB::enable_profile() and DB::disable_profile() yourself.
 
 =head2 subs=0
 
-Set to 0 to disable the collection of subroutine inclusive timings.
+Set to 0 to disable the collection of subroutine caller and timing details.
 
 =head2 blocks=0
 
 Set to 0 to disable the determination of block and subroutine location per statement.
-This makes the profiler about 50% faster (as of July 2008) but you loose some
-valuable information. The extra cost is likely to be reduced in later versions
-anyway, as little optimization has been done on that part of the code.
-The profiler is fast enough that you shouldn't need to do this.
+This makes the profiler about 50% faster (as of July 2008) and produces smaller
+output files, but you loose some valuable information. The extra cost is likely
+to be reduced in later versions anyway, as little optimization has been done on
+that part of the code.
+
+=head2 stmts=0
+
+Set to 0 to disable the statement profiler. (Implies C<blocks=0>.)
+The reports won't contain any statement timing detail.
+
+This significantly reduces the overhead of the profiler and can also be useful
+for profiling large applications that would normally generate a very large
+profile data file.
 
 =head2 leave=0
 
@@ -273,6 +340,9 @@ gigahertz clocks, 0.01 seconds is like a lifetime. The cpu time clock 'ticks'
 happen so rarely relative to the activity of a most applications that you'd
 have to run the code for many hours to have any hope of reasonably useful results.
 
+A better alternative would be to use the C<clock=N> option to select a
+high-resolution cpu time clock, if available on your system.
+
 =head2 file=...
 
 Specify the output file to write profile data to (default: './nytprof.out').
@@ -290,21 +360,25 @@ If NYTProf was not compiled with compression support, this option is silently ig
 
 Systems which support the C<clock_gettime()> system call typically
 support several clocks. By default NYTProf uses CLOCK_MONOTONIC.
+
 This option enables you to select a different clock by specifying the
-integer id of the clock (which may vary between systems). If the clock
-you select isn't available then CLOCK_REALTIME is used.
+integer id of the clock (which may vary between operating system types).
+If the clock you select isn't available then CLOCK_REALTIME is used.
 
-This is a wizardly option and best avoided unless you really know what
-you're doing and understand the subtle differences between the clocks.
+See L</CLOCKS> for more information.
 
-=head1 SELECTIVE PROFILING
+=head1 RUN-TIME CONTROL OF PROFILING
 
 You can profile only parts of an application by calling DB::enable_profile()
 and DB::disable_profile() at the appropriate moments.
 
-Using the C<start=no> option let's you leave the profiler disabled until the
+Using the C<start=no> option lets you leave the profiler disabled until the
 right moment, or circumstances, are reached.
 
+You can finish profiling completely by calling DB::finish_profile().
+This may be useful if perl is exiting abnormally, leaving the profile data file
+in an incomplete state,
+
 =head1 REPORTS
 
 The L<Devel::NYTProf::Data> module provides a low-level interface for loading
@@ -317,53 +391,225 @@ perl. (Though the module is in a state of flux and may be deprecated soon.)
 Included in the bin directory of this distribution are two scripts
 which implement the L<Devel::NYTProf::Reader> interface: 
 
-=over 4
+=over 12
 
-=item * 
-nytprofcsv - creates comma delimited profile reports
+=item nytprofcsv
 
-=item *
-nytprofhtml - creates attractive, richly annotated, and fully cross-linked html
+creates comma delimited profile reports
+
+=item nytprofhtml
+
+creates attractive, richly annotated, and fully cross-linked html
 reports (including statistics, source code and color highlighting)
 
 =back
 
+=head1 CLOCKS
+
+Here we discuss the way NYTProf gets high-resolution timing information from
+your system and related issues.
+
+=head2 POSIX Clocks
+
+These are the clocks that your system may support if it supports the POSIX
+C<clock_gettime()> function. Other clock sources are listed in the
+L</Other Clocks> section below.
+
+The C<clock_gettime()> interface allows clocks to return times to nanosecond
+precision. Of course few offer nanosecond I<accuracy> but the extra precision
+helps reduce the cumulative error that naturally occurs when adding together
+many timings. When using these clocks NYTProf outputs timings as a count of 100
+nanosecond ticks.
+
+=head3 CLOCK_REALTIME
+
+CLOCK_REALTIME is typically the system's main high resolution 'wall clock time'
+source.  The same source as used for the gettimeofday() call used by most kinds
+of perl benchmarking and profiling tools.
+
+If your system doesn't support clock_gettime() then NYTProf will use
+gettimeofday(), or the nearest equivalent,
+
+The problem with real time is that it's far from simple. It tends to drift and
+then be reset to match 'reality', either sharply or by small adjustments (via the
+adjtime() system call).
+
+Surprizingly, it can also go backwards, for reasons explained in
+http://preview.tinyurl.com/5wawnn
+
+=head3 CLOCK_MONOTONIC
+
+CLOCK_MONOTONIC rrepresents the amount of time since an unspecified point in
+the past (typically system start-up time).  It increments uniformally
+independent of adjustments to 'wallclock time'.
+
+=head3 CLOCK_VIRTUAL
+
+CLOCK_VIRTUAL increments only when the CPU is running in user mode on behalf of the calling process.
+
+=head3 CLOCK_PROF
+
+CLOCK_PROF increments when the CPU is running in user I<or> kernel mode.
+
+=head3 CLOCK_PROCESS_CPUTIME_ID
+
+CLOCK_PROCESS_CPUTIME_ID represents the amount of execution time of the process associated with the clock.
+
+=head3 CLOCK_THREAD_CPUTIME_ID
+
+CLOCK_THREAD_CPUTIME_ID represents the amount of execution time of the thread associated with the clock.
+
+=head3 Finding Available POSIX Clocks
+
+On unix-like systems you can find the CLOCK_* clocks available on you system
+using a command like:
+
+  grep -r 'define *CLOCK_' /usr/include
+
+Look for a group that includes CLOCK_REALTIME. The integer values listed are
+the clock ids that you can use with the C<clock=N> option.
+
+A future version of NYTProf should be able to list the supported clocks.
+
+=head2 Other Clocks
+
+This section lists other clock sources that NYTProf may use.
+
+=head3 gettimeofday
+
+This is the traditional high resolution time of day interface for most
+unix-like systems. It's used on platforms like Mac OS X which don't
+(yet) support C<clock_gettime()>.
+
+With this clock NYTProf outputs timings as a count of 1 microsecond ticks.
+
+=for comment re high resolution timing for OS X:
+http://developer.apple.com/qa/qa2004/qa1398.html
+http://www.macresearch.org/tutorial_performance_and_time
+http://cocoasamurai.blogspot.com/2006/12/tip-when-you-must-be-precise-be-mach.html
+http://boredzo.org/blog/archives/2006-11-26/how-to-use-mach-clocks
+
+=head3 Time::HiRes
+
+On systems which don't support C<clock_gettime()> or C<gettimeofday()>
+NYTProf falls back to using the L<Time::HiRes> module.
+With this clock NYTProf outputs timings as a count of 1 microsecond ticks.
+
+=head2 Clock References
+
+Relevant specifications and manual pages:
+
+  http://www.opengroup.org/onlinepubs/000095399/functions/clock_getres.html
+  http://linux.die.net/man/3/clock_gettime
+
+Why 'realtime' can appear to go backwards:
+
+  http://preview.tinyurl.com/5wawnn
+
+=for comment
+http://preview.tinyurl.com/5wawnn redirects to:
+http://groups.google.com/group/comp.os.linux.development.apps/tree/browse_frm/thread/dc29071f2417f75f/ac44671fdb35f6db?rnum=1&_done=%2Fgroup%2Fcomp.os.linux.development.apps%2Fbrowse_frm%2Fthread%2Fdc29071f2417f75f%2Fc46264dba0863463%3Flnk%3Dst%26rnum%3D1%26
+
+=for comment - these links seem broken
+http://webnews.giga.net.tw/article//mailing.freebsd.performance/710
+http://sean.chittenden.org/news/2008/06/01/
+
 =head1 LIMITATIONS
 
-=head2 Only profiles code loaded after this module
-
-Loading via the perl -d option ensures it's loaded first.
-
 =head2 threads
 
 C<Devel::NYTProf> is not currently thread safe. If you'd be interested in
-helping us make it thread safe then please get in touch with us.
+helping to make it thread safe then please get in touch with us.
 
 =head2 For perl versions before 5.8.8 it may change what caller() returns
 
 For example, the Readonly module croaks with an "Invalid tie" when profiled with
 perl versions before 5.8.8. That's because L<Readonly> explicitly checking for
-certain values from caller().  We're not quite sure what the cause is yet.
+certain values from caller().
 
 =head2 Calls made via operator overloading
 
 Calls made via operator overloading are not noticed by any subroutine profiler.
+Though the statements executed by the code in the overload subs are profiled.
 
 =head2 goto
 
-The C<goto &$sub;> isn't recognised as a subroutine call by the subroutine profiler.
-
-=head2 Windows
-
-Currently there's no support for Windows. Some work is being done on a port.
-If you'd be interested in helping us port to Windows then please get in touch
-with us.
+The C<goto &foo;> isn't recognised as a subroutine call by the subroutine profiler.
 
 =head2 #line directives
 
 The reporting code currently doesn't handle #line directives, but at least it
 warns about them. Patches welcome.
 
+=head1 CAVEATS
+
+=head2 SMP Systems
+
+Systems with multiple processors, which includes most modern machines, have
+
+From Linux docs (though applicable to most SMP systems):
+
+  The CLOCK_PROCESS_CPUTIME_ID and CLOCK_THREAD_CPUTIME_ID clocks are realized on
+  many platforms using timers from the CPUs (TSC on i386, AR.ITC on Itanium).
+  These registers may differ between CPUs and as a consequence these clocks may
+  return bogus results if a process is migrated to another CPU.
+
+  If the CPUs in an SMP system have different clock sources then there is no way
+  to maintain a correlation between the timer registers since each CPU will run
+  at a slightly different frequency. If that is the case then
+  clock_getcpuclockid(0) will return ENOENT to signify this condition. The two
+  clocks will then only be useful if it can be ensured that a process stays on a
+  certain CPU.
+
+  The processors in an SMP system do not start all at exactly the same time and
+  therefore the timer registers are typically running at an offset. Some
+  architectures include code that attempts to limit these offsets on bootup.
+  However, the code cannot guarantee to accurately tune the offsets. Glibc
+  contains no provisions to deal with these offsets (unlike the Linux Kernel).
+  Typically these offsets are small and therefore the effects may be negligible
+  in most cases.
+
+In summary, SMP systems are likely to give 'noisy' profiles.
+Setting a L<Processor Affinity> may help.
+
+=head3 Processor Affinity
+
+Processor affinity is an aspect of task scheduling on SMP systems.
+"Processor affinity takes advantage of the fact that some remnants of a process
+may remain in one processor's state (in particular, in its cache) from the last
+time the process ran, and so scheduling it to run on the same processor the
+next time could result in the process running more efficiently than if it were
+to run on another processor." (From http://en.wikipedia.org/wiki/Processor_affinity)
+
+Setting an explicit processor affinity can avoid the problems described in
+L</SMP Systems>.
+
+Processor affinity can be set using the C<taskset> command on Linux.
+
+Future versions of NYTProf could support setting processor affinity automatically
+(e.g. via sched_setaffinity() on Linux). Patches welcome!
+
+Note that processor affinity is inherited by child processes, so if the process
+you're profiling spawns cpu intensive sub processes then your process will be
+impacted by those more than it otherwise would.
+
+=head2 Virtual Machines
+
+I recommend you don't do performance profiling while running in a
+virtual machine.  If you do you're likely to find inexplicable spikes
+of real-time appearing at unreasonable places in your code. You should pay
+less attention to the statement timings and rely more on the subroutine
+timings. They will still be noisy but less so than the statement times.
+
+You could also try using the C<clock=N> option to select a high-resolution
+I<cpu-time> clock instead of a real-time one. That should be much less
+noisy, though you will loose visibility of wait-times due to network
+and disk I/O, for example.
+
+If your system doesn't support the C<clock=N> option then you could try
+using the C<usecputime=1> option. That will give you cpu-time measurements
+but only at a very low 1/100th of a second resolution.
+
 =head1 BUGS
 
 Possibly.
@@ -380,6 +626,8 @@ L<http://blog.timbunce.org/2008/07/16/nytprof-v2-the-background-story/>.
 
 Mailing list and discussion at L<http://groups.google.com/group/develnytprof-dev>
 
+Blog posts L<http://blog.timbunce.org/tag/nytprof/> and L<http://technorati.com/search/nytprof>
+
 Public SVN Repository and hacking instructions at L<http://code.google.com/p/perl-devel-nytprof/>
 
 L<nytprofhtml> is a script included that produces html reports.
@@ -387,7 +635,10 @@ L<nytprofcsv> is another script included that produces plain text CSV reports.
 
 L<Devel::NYTProf::Reader> is the module that powers the report scripts.  You
 might want to check this out if you plan to implement a custom report (though
-it may be deprecated in a future release).
+it's very likely to be deprecated in a future release).
+
+L<Devel::NYTProf::ReadStream> is the module that lets you read a profile data
+file as a stream of chunks of data.
 
 =head1 AUTHOR
 
@@ -456,7 +707,8 @@ to display all the extra profile call and timing data in richly annotated and
 cross-linked reports.
 
 Steve Peters came on board along the way with patches for portability and to
-keep NYTProf working with the latest development perl versions.
+keep NYTProf working with the latest development perl versions. Nicholas Clark
+added zip compression. Jan Dubois contributed Windows support.
 
 Adam's work is sponsored by The New York Times Co. L<http://open.nytimes.com>.
 Tim's work was partly sponsored by Shopzilla. L<http://www.shopzilla.com>.
